@@ -27,12 +27,21 @@
  */
 
 /* \summary: Babel Routing Protocol printer */
+/* Specifications:
+ *
+ * RFC 6126
+ * RFC 7298
+ * RFC 7557
+ * draft-ietf-babel-rfc6126bis-17
+ * draft-ietf-babel-hmac-10
+ * draft-ietf-babel-source-specific-0
+ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -41,39 +50,37 @@
 #include "addrtoname.h"
 #include "extract.h"
 
-static const char tstr[] = "[|babel]";
-
 static void babel_print_v2(netdissect_options *, const u_char *cp, u_int length);
 
 void
 babel_print(netdissect_options *ndo,
             const u_char *cp, u_int length)
 {
-    ND_PRINT((ndo, "babel"));
+    ndo->ndo_protocol = "babel";
+    ND_PRINT("babel");
 
-    ND_TCHECK2(*cp, 4);
+    ND_TCHECK_4(cp);
 
-    if(cp[0] != 42) {
-        ND_PRINT((ndo, " invalid header"));
+    if(GET_U_1(cp) != 42) {
+        ND_PRINT(" invalid header");
         return;
     } else {
-        ND_PRINT((ndo, " %d", cp[1]));
+        ND_PRINT(" %u", GET_U_1(cp + 1));
     }
 
-    switch(cp[1]) {
+    switch(GET_U_1(cp + 1)) {
     case 2:
         babel_print_v2(ndo, cp, length);
         break;
     default:
-        ND_PRINT((ndo, " unknown version"));
+        ND_PRINT(" unknown version");
         break;
     }
 
     return;
 
  trunc:
-    ND_PRINT((ndo, " %s", tstr));
-    return;
+    nd_print_trunc(ndo);
 }
 
 /* TLVs */
@@ -86,13 +93,17 @@ babel_print(netdissect_options *ndo,
 #define MESSAGE_ROUTER_ID 6
 #define MESSAGE_NH 7
 #define MESSAGE_UPDATE 8
-#define MESSAGE_REQUEST 9
-#define MESSAGE_MH_REQUEST 10
+#define MESSAGE_ROUTE_REQUEST 9
+#define MESSAGE_SEQNO_REQUEST 10
 #define MESSAGE_TSPC 11
 #define MESSAGE_HMAC 12
-#define MESSAGE_UPDATE_SRC_SPECIFIC 13
-#define MESSAGE_REQUEST_SRC_SPECIFIC 14
-#define MESSAGE_MH_REQUEST_SRC_SPECIFIC 15
+#define MESSAGE_UPDATE_SRC_SPECIFIC 13 /* last appearance in draft-boutier-babel-source-specific-01 */
+#define MESSAGE_REQUEST_SRC_SPECIFIC 14 /* idem */
+#define MESSAGE_MH_REQUEST_SRC_SPECIFIC 15 /* idem */
+#define MESSAGE_MAC 16
+#define MESSAGE_PC 17
+#define MESSAGE_CHALLENGE_REQUEST 18
+#define MESSAGE_CHALLENGE_REPLY 19
 
 /* sub-TLVs */
 #define MESSAGE_SUB_PAD1 0
@@ -100,6 +111,12 @@ babel_print(netdissect_options *ndo,
 #define MESSAGE_SUB_DIVERSITY 2
 #define MESSAGE_SUB_TIMESTAMP 3
 
+/* "Mandatory" bit in sub-TLV types */
+#define MANDATORY_MASK 0x80
+
+/* Flags for the Hello TLV */
+#define UNICAST_MASK 0x8000
+
 /* Diversity sub-TLV channel codes */
 static const struct tok diversity_str[] = {
     { 0,   "reserved" },
@@ -108,11 +125,13 @@ static const struct tok diversity_str[] = {
 };
 
 static const char *
-format_id(const u_char *id)
+format_id(netdissect_options *ndo, const u_char *id)
 {
     static char buf[25];
     snprintf(buf, 25, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x",
-             id[0], id[1], id[2], id[3], id[4], id[5], id[6], id[7]);
+             GET_U_1(id), GET_U_1(id + 1), GET_U_1(id + 2),
+             GET_U_1(id + 3), GET_U_1(id + 4), GET_U_1(id + 5),
+             GET_U_1(id + 6), GET_U_1(id + 7));
     buf[24] = '\0';
     return buf;
 }
@@ -124,6 +143,12 @@ static const char *
 format_prefix(netdissect_options *ndo, const u_char *prefix, unsigned char plen)
 {
     static char buf[50];
+
+    /*
+     * prefix points to a buffer on the stack into which the prefix has
+     * been placed, so we can't use GET_IPADDR_STRING() or
+     * GET_IP6ADDR_STRING() on it.
+     */
     if(plen >= 96 && memcmp(prefix, v4prefix, 12) == 0)
         snprintf(buf, 50, "%s/%u", ipaddr_string(ndo, prefix + 12), plen - 96);
     else
@@ -135,6 +160,11 @@ format_prefix(netdissect_options *ndo, const u_char *prefix, unsigned char plen)
 static const char *
 format_address(netdissect_options *ndo, const u_char *prefix)
 {
+    /*
+     * prefix points to a buffer on the stack into which the prefix has
+     * been placed, so we can't use GET_IPADDR_STRING() or
+     * GET_IP6ADDR_STRING() on it.
+     */
     if(memcmp(prefix, v4prefix, 12) == 0)
         return ipaddr_string(ndo, prefix + 12);
     else
@@ -249,7 +279,7 @@ network_address(int ae, const unsigned char *a, unsigned int len,
  * o Type 1 stands for PadN sub-TLV with the same encoding as the PadN TLV.
  * o Type 2 stands for Diversity sub-TLV, which propagates diversity routing
  *   data. Its body is a variable-length sequence of 8-bit unsigned integers,
- *   each representing per-hop number of interferring radio channel for the
+ *   each representing per-hop number of interfering radio channel for the
  *   prefix. Channel 0 is invalid and must not be used in the sub-TLV, channel
  *   255 interferes with any other channel.
  * o Type 3 stands for Timestamp sub-TLV, used to compute RTT between
@@ -274,128 +304,128 @@ subtlvs_print(netdissect_options *ndo,
     uint32_t t1, t2;
 
     while (cp < ep) {
-        subtype = *cp++;
+        subtype = GET_U_1(cp);
+        cp++;
         if(subtype == MESSAGE_SUB_PAD1) {
-            ND_PRINT((ndo, " sub-pad1"));
+            ND_PRINT(" sub-pad1");
             continue;
         }
+        if ((MANDATORY_MASK & subtype) != 0)
+            ND_PRINT(" (M)");
         if(cp == ep)
             goto invalid;
-        sublen = *cp++;
+        sublen = GET_U_1(cp);
+        cp++;
         if(cp + sublen > ep)
             goto invalid;
 
         switch(subtype) {
         case MESSAGE_SUB_PADN:
-            ND_PRINT((ndo, " sub-padn"));
+            ND_PRINT(" sub-padn");
             cp += sublen;
             break;
         case MESSAGE_SUB_DIVERSITY:
-            ND_PRINT((ndo, " sub-diversity"));
+            ND_PRINT(" sub-diversity");
             if (sublen == 0) {
-                ND_PRINT((ndo, " empty"));
+                ND_PRINT(" empty");
                 break;
             }
             sep = " ";
-            while(sublen--) {
-                ND_PRINT((ndo, "%s%s", sep, tok2str(diversity_str, "%u", *cp++)));
+            while (sublen) {
+                ND_PRINT("%s%s", sep,
+                         tok2str(diversity_str, "%u", GET_U_1(cp)));
+                cp++;
                 sep = "-";
+                sublen--;
             }
             if(tlv_type != MESSAGE_UPDATE &&
                tlv_type != MESSAGE_UPDATE_SRC_SPECIFIC)
-                ND_PRINT((ndo, " (bogus)"));
+                ND_PRINT(" (bogus)");
             break;
         case MESSAGE_SUB_TIMESTAMP:
-            ND_PRINT((ndo, " sub-timestamp"));
+            ND_PRINT(" sub-timestamp");
             if(tlv_type == MESSAGE_HELLO) {
                 if(sublen < 4)
                     goto invalid;
-                t1 = EXTRACT_32BITS(cp);
-                ND_PRINT((ndo, " %s", format_timestamp(t1)));
+                t1 = GET_BE_U_4(cp);
+                ND_PRINT(" %s", format_timestamp(t1));
             } else if(tlv_type == MESSAGE_IHU) {
                 if(sublen < 8)
                     goto invalid;
-                t1 = EXTRACT_32BITS(cp);
-                ND_PRINT((ndo, " %s", format_timestamp(t1)));
-                t2 = EXTRACT_32BITS(cp + 4);
-                ND_PRINT((ndo, "|%s", format_timestamp(t2)));
+                t1 = GET_BE_U_4(cp);
+                ND_PRINT(" %s", format_timestamp(t1));
+                t2 = GET_BE_U_4(cp + 4);
+                ND_PRINT("|%s", format_timestamp(t2));
             } else
-                ND_PRINT((ndo, " (bogus)"));
+                ND_PRINT(" (bogus)");
             cp += sublen;
             break;
         default:
-            ND_PRINT((ndo, " sub-unknown-0x%02x", subtype));
+            ND_PRINT(" sub-unknown-0x%02x", subtype);
             cp += sublen;
         } /* switch */
     } /* while */
     return;
 
  invalid:
-    ND_PRINT((ndo, "%s", istr));
+    nd_print_invalid(ndo);
 }
 
 #define ICHECK(i, l) \
-	if ((i) + (l) > bodylen || (i) + (l) > length) goto invalid;
+	if ((i) + (l) > tlvs_length || (i) + (l) > packet_length_remaining) \
+	    goto invalid;
 
-static void
-babel_print_v2(netdissect_options *ndo,
-               const u_char *cp, u_int length)
+static int
+babel_print_v2_tlvs(netdissect_options *ndo,
+                    const u_char *cp, u_int tlvs_length,
+                    u_int packet_length_remaining)
 {
     u_int i;
-    u_short bodylen;
     u_char v4_prefix[16] =
         {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xFF, 0xFF, 0, 0, 0, 0 };
     u_char v6_prefix[16] = {0};
 
-    ND_TCHECK2(*cp, 4);
-    if (length < 4)
-        goto invalid;
-    bodylen = EXTRACT_16BITS(cp + 2);
-    ND_PRINT((ndo, " (%u)", bodylen));
-    if (4U + bodylen > length)
-        goto invalid;
-
-    /* Process the TLVs in the body */
     i = 0;
-    while(i < bodylen) {
+    while(i < tlvs_length) {
         const u_char *message;
-        u_int type, len;
+        uint8_t type;
+        u_int len;
 
-        message = cp + 4 + i;
+        message = cp + i;
 
-        ND_TCHECK2(*message, 1);
-        if((type = message[0]) == MESSAGE_PAD1) {
-            ND_PRINT((ndo, ndo->ndo_vflag ? "\n\tPad 1" : " pad1"));
+        ICHECK(i, 1);
+        if((type = GET_U_1(message)) == MESSAGE_PAD1) {
+            ND_PRINT(ndo->ndo_vflag ? "\n\tPad 1" : " pad1");
             i += 1;
             continue;
         }
 
-        ND_TCHECK2(*message, 2);
         ICHECK(i, 2);
-        len = message[1];
+        ND_TCHECK_2(message);
+        len = GET_U_1(message + 1);
 
-        ND_TCHECK2(*message, 2 + len);
         ICHECK(i, 2 + len);
+        ND_TCHECK_LEN(message, 2 + len);
 
         switch(type) {
         case MESSAGE_PADN: {
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " padN"));
+                ND_PRINT(" padN");
             else
-                ND_PRINT((ndo, "\n\tPad %d", len + 2));
+                ND_PRINT("\n\tPad %u", len + 2);
         }
             break;
 
         case MESSAGE_ACK_REQ: {
             u_short nonce, interval;
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " ack-req"));
+                ND_PRINT(" ack-req");
             else {
-                ND_PRINT((ndo, "\n\tAcknowledgment Request "));
+                ND_PRINT("\n\tAcknowledgment Request ");
                 if(len < 6) goto invalid;
-                nonce = EXTRACT_16BITS(message + 4);
-                interval = EXTRACT_16BITS(message + 6);
-                ND_PRINT((ndo, "%04x %s", nonce, format_interval(interval)));
+                nonce = GET_BE_U_2(message + 4);
+                interval = GET_BE_U_2(message + 6);
+                ND_PRINT("%04x %s", nonce, format_interval(interval));
             }
         }
             break;
@@ -403,26 +433,33 @@ babel_print_v2(netdissect_options *ndo,
         case MESSAGE_ACK: {
             u_short nonce;
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " ack"));
+                ND_PRINT(" ack");
             else {
-                ND_PRINT((ndo, "\n\tAcknowledgment "));
+                ND_PRINT("\n\tAcknowledgment ");
                 if(len < 2) goto invalid;
-                nonce = EXTRACT_16BITS(message + 2);
-                ND_PRINT((ndo, "%04x", nonce));
+                nonce = GET_BE_U_2(message + 2);
+                ND_PRINT("%04x", nonce);
             }
         }
             break;
 
         case MESSAGE_HELLO:  {
-            u_short seqno, interval;
+            u_short seqno, interval, unicast;
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " hello"));
+                ND_PRINT(" hello");
             else {
-                ND_PRINT((ndo, "\n\tHello "));
+                ND_PRINT("\n\tHello ");
                 if(len < 6) goto invalid;
-                seqno = EXTRACT_16BITS(message + 4);
-                interval = EXTRACT_16BITS(message + 6);
-                ND_PRINT((ndo, "seqno %u interval %s", seqno, format_interval(interval)));
+                unicast = (GET_BE_U_2(message + 2) & UNICAST_MASK);
+                seqno = GET_BE_U_2(message + 4);
+                interval = GET_BE_U_2(message + 6);
+                if(unicast)
+                     ND_PRINT("(Unicast) ");
+                ND_PRINT("seqno %u ", seqno);
+                if(interval!=0)
+                    ND_PRINT("interval %s", format_interval(interval));
+                else
+                    ND_PRINT("unscheduled");
                 /* Extra data. */
                 if(len > 6)
                     subtlvs_print(ndo, message + 8, message + 2 + len, type);
@@ -431,20 +468,24 @@ babel_print_v2(netdissect_options *ndo,
             break;
 
         case MESSAGE_IHU: {
-            unsigned short txcost, interval;
+            unsigned short rxcost, interval;
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " ihu"));
+                ND_PRINT(" ihu");
             else {
                 u_char address[16];
+                u_char ae;
                 int rc;
-                ND_PRINT((ndo, "\n\tIHU "));
+                ND_PRINT("\n\tIHU ");
                 if(len < 6) goto invalid;
-                txcost = EXTRACT_16BITS(message + 4);
-                interval = EXTRACT_16BITS(message + 6);
-                rc = network_address(message[2], message + 8, len - 6, address);
-                if(rc < 0) { ND_PRINT((ndo, "%s", tstr)); break; }
-                ND_PRINT((ndo, "%s txcost %u interval %s",
-                       format_address(ndo, address), txcost, format_interval(interval)));
+                rxcost = GET_BE_U_2(message + 4);
+                interval = GET_BE_U_2(message + 6);
+                ae = GET_U_1(message + 2);
+                rc = network_address(ae, message + 8,
+                                     len - 6, address);
+                if(rc < 0) { nd_print_trunc(ndo); break; }
+                ND_PRINT("%s rxcost %u interval %s",
+                       ae == 0 ? "any" : format_address(ndo, address),
+                       rxcost, format_interval(interval));
                 /* Extra data. */
                 if((u_int)rc < len - 6)
                     subtlvs_print(ndo, message + 8 + rc, message + 2 + len,
@@ -455,64 +496,70 @@ babel_print_v2(netdissect_options *ndo,
 
         case MESSAGE_ROUTER_ID: {
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " router-id"));
+                ND_PRINT(" router-id");
             else {
-                ND_PRINT((ndo, "\n\tRouter Id"));
+                ND_PRINT("\n\tRouter Id");
                 if(len < 10) goto invalid;
-                ND_PRINT((ndo, " %s", format_id(message + 4)));
+                ND_PRINT(" %s", format_id(ndo, message + 4));
             }
         }
             break;
 
         case MESSAGE_NH: {
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " nh"));
+                ND_PRINT(" nh");
             else {
                 int rc;
+                u_char ae;
                 u_char nh[16];
-                ND_PRINT((ndo, "\n\tNext Hop"));
+                ND_PRINT("\n\tNext Hop");
                 if(len < 2) goto invalid;
-                rc = network_address(message[2], message + 4, len - 2, nh);
+                ae = GET_U_1(message + 2);
+                rc = network_address(ae, message + 4,
+                                     len - 2, nh);
                 if(rc < 0) goto invalid;
-                ND_PRINT((ndo, " %s", format_address(ndo, nh)));
+                ND_PRINT(" %s", ae == 0 ? "invalid AE 0" : format_address(ndo, nh));
             }
         }
             break;
 
         case MESSAGE_UPDATE: {
             if (!ndo->ndo_vflag) {
-                ND_PRINT((ndo, " update"));
+                ND_PRINT(" update");
                 if(len < 10)
-                    ND_PRINT((ndo, "/truncated"));
+                    goto invalid;
                 else
-                    ND_PRINT((ndo, "%s%s%s",
-                           (message[3] & 0x80) ? "/prefix": "",
-                           (message[3] & 0x40) ? "/id" : "",
-                           (message[3] & 0x3f) ? "/unknown" : ""));
+                    ND_PRINT("%s%s%s",
+                           (GET_U_1(message + 3) & 0x80) ? "/prefix": "",
+                           (GET_U_1(message + 3) & 0x40) ? "/id" : "",
+                           (GET_U_1(message + 3) & 0x3f) ? "/unknown" : "");
             } else {
                 u_short interval, seqno, metric;
-                u_char plen;
+                u_char ae, plen;
                 int rc;
                 u_char prefix[16];
-                ND_PRINT((ndo, "\n\tUpdate"));
+                ND_PRINT("\n\tUpdate");
                 if(len < 10) goto invalid;
-                plen = message[4] + (message[2] == 1 ? 96 : 0);
-                rc = network_prefix(message[2], message[4], message[5],
+                ae = GET_U_1(message + 2);
+                plen = GET_U_1(message + 4) + (GET_U_1(message + 2) == 1 ? 96 : 0);
+                rc = network_prefix(ae,
+                                    GET_U_1(message + 4),
+                                    GET_U_1(message + 5),
                                     message + 12,
-                                    message[2] == 1 ? v4_prefix : v6_prefix,
+                                    GET_U_1(message + 2) == 1 ? v4_prefix : v6_prefix,
                                     len - 10, prefix);
                 if(rc < 0) goto invalid;
-                interval = EXTRACT_16BITS(message + 6);
-                seqno = EXTRACT_16BITS(message + 8);
-                metric = EXTRACT_16BITS(message + 10);
-                ND_PRINT((ndo, "%s%s%s %s metric %u seqno %u interval %s",
-                       (message[3] & 0x80) ? "/prefix": "",
-                       (message[3] & 0x40) ? "/id" : "",
-                       (message[3] & 0x3f) ? "/unknown" : "",
-                       format_prefix(ndo, prefix, plen),
-                       metric, seqno, format_interval_update(interval)));
-                if(message[3] & 0x80) {
-                    if(message[2] == 1)
+                interval = GET_BE_U_2(message + 6);
+                seqno = GET_BE_U_2(message + 8);
+                metric = GET_BE_U_2(message + 10);
+                ND_PRINT("%s%s%s %s metric %u seqno %u interval %s",
+                       (GET_U_1(message + 3) & 0x80) ? "/prefix": "",
+                       (GET_U_1(message + 3) & 0x40) ? "/id" : "",
+                       (GET_U_1(message + 3) & 0x3f) ? "/unknown" : "",
+                       ae == 0 ? "any" : format_prefix(ndo, prefix, plen),
+                       metric, seqno, format_interval_update(interval));
+                if(GET_U_1(message + 3) & 0x80) {
+                    if(GET_U_1(message + 2) == 1)
                         memcpy(v4_prefix, prefix, 16);
                     else
                         memcpy(v6_prefix, prefix, 16);
@@ -524,86 +571,93 @@ babel_print_v2(netdissect_options *ndo,
         }
             break;
 
-        case MESSAGE_REQUEST: {
+        case MESSAGE_ROUTE_REQUEST: {
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " request"));
+                ND_PRINT(" route-request");
             else {
                 int rc;
-                u_char prefix[16], plen;
-                ND_PRINT((ndo, "\n\tRequest "));
+                u_char prefix[16], ae, plen;
+                ND_PRINT("\n\tRoute Request ");
                 if(len < 2) goto invalid;
-                plen = message[3] + (message[2] == 1 ? 96 : 0);
-                rc = network_prefix(message[2], message[3], 0,
+                ae = GET_U_1(message + 2);
+                plen = GET_U_1(message + 3) + (GET_U_1(message + 2) == 1 ? 96 : 0);
+                rc = network_prefix(ae,
+                                    GET_U_1(message + 3), 0,
                                     message + 4, NULL, len - 2, prefix);
                 if(rc < 0) goto invalid;
-                ND_PRINT((ndo, "for %s",
-                       message[2] == 0 ? "any" : format_prefix(ndo, prefix, plen)));
+                ND_PRINT("for %s",
+                       ae == 0 ? "any" : format_prefix(ndo, prefix, plen));
             }
         }
             break;
 
-        case MESSAGE_MH_REQUEST : {
+        case MESSAGE_SEQNO_REQUEST : {
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " mh-request"));
+                ND_PRINT(" seqno-request");
             else {
                 int rc;
                 u_short seqno;
-                u_char prefix[16], plen;
-                ND_PRINT((ndo, "\n\tMH-Request "));
+                u_char prefix[16], ae, plen;
+                ND_PRINT("\n\tSeqno Request ");
                 if(len < 14) goto invalid;
-                seqno = EXTRACT_16BITS(message + 4);
-                rc = network_prefix(message[2], message[3], 0,
+                ae = GET_U_1(message + 2);
+                seqno = GET_BE_U_2(message + 4);
+                rc = network_prefix(ae,
+                                    GET_U_1(message + 3), 0,
                                     message + 16, NULL, len - 14, prefix);
                 if(rc < 0) goto invalid;
-                plen = message[3] + (message[2] == 1 ? 96 : 0);
-                ND_PRINT((ndo, "(%u hops) for %s seqno %u id %s",
-                       message[6], format_prefix(ndo, prefix, plen),
-                       seqno, format_id(message + 8)));
+                plen = GET_U_1(message + 3) + (GET_U_1(message + 2) == 1 ? 96 : 0);
+                ND_PRINT("(%u hops) for %s seqno %u id %s",
+                       GET_U_1(message + 6),
+                       ae == 0 ? "invalid AE 0" : format_prefix(ndo, prefix, plen),
+                       seqno, format_id(ndo, message + 8));
             }
         }
             break;
         case MESSAGE_TSPC :
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " tspc"));
+                ND_PRINT(" tspc");
             else {
-                ND_PRINT((ndo, "\n\tTS/PC "));
+                ND_PRINT("\n\tTS/PC ");
                 if(len < 6) goto invalid;
-                ND_PRINT((ndo, "timestamp %u packetcounter %u", EXTRACT_32BITS (message + 4),
-                       EXTRACT_16BITS(message + 2)));
+                ND_PRINT("timestamp %u packetcounter %u",
+                          GET_BE_U_4(message + 4),
+                          GET_BE_U_2(message + 2));
             }
             break;
         case MESSAGE_HMAC : {
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " hmac"));
+                ND_PRINT(" hmac");
             else {
                 unsigned j;
-                ND_PRINT((ndo, "\n\tHMAC "));
+                ND_PRINT("\n\tHMAC ");
                 if(len < 18) goto invalid;
-                ND_PRINT((ndo, "key-id %u digest-%u ", EXTRACT_16BITS(message + 2), len - 2));
+                ND_PRINT("key-id %u digest-%u ", GET_BE_U_2(message + 2),
+                         len - 2);
                 for (j = 0; j < len - 2; j++)
-                    ND_PRINT((ndo, "%02X", message[4 + j]));
+                    ND_PRINT("%02X", GET_U_1(message + j + 4));
             }
         }
             break;
 
         case MESSAGE_UPDATE_SRC_SPECIFIC : {
             if(!ndo->ndo_vflag) {
-                ND_PRINT((ndo, " ss-update"));
+                ND_PRINT(" ss-update");
             } else {
                 u_char prefix[16], src_prefix[16];
                 u_short interval, seqno, metric;
                 u_char ae, plen, src_plen, omitted;
                 int rc;
                 int parsed_len = 10;
-                ND_PRINT((ndo, "\n\tSS-Update"));
+                ND_PRINT("\n\tSS-Update");
                 if(len < 10) goto invalid;
-                ae = message[2];
-                src_plen = message[3];
-                plen = message[4];
-                omitted = message[5];
-                interval = EXTRACT_16BITS(message + 6);
-                seqno = EXTRACT_16BITS(message + 8);
-                metric = EXTRACT_16BITS(message + 10);
+                ae = GET_U_1(message + 2);
+                src_plen = GET_U_1(message + 3);
+                plen = GET_U_1(message + 4);
+                omitted = GET_U_1(message + 5);
+                interval = GET_BE_U_2(message + 6);
+                seqno = GET_BE_U_2(message + 8);
+                metric = GET_BE_U_2(message + 10);
                 rc = network_prefix(ae, plen, omitted, message + 2 + parsed_len,
                                     ae == 1 ? v4_prefix : v6_prefix,
                                     len - parsed_len, prefix);
@@ -618,10 +672,10 @@ babel_print_v2(netdissect_options *ndo,
                     src_plen += 96;
                 parsed_len += rc;
 
-                ND_PRINT((ndo, " %s from", format_prefix(ndo, prefix, plen)));
-                ND_PRINT((ndo, " %s metric %u seqno %u interval %s",
+                ND_PRINT(" %s from", format_prefix(ndo, prefix, plen));
+                ND_PRINT(" %s metric %u seqno %u interval %s",
                           format_prefix(ndo, src_prefix, src_plen),
-                          metric, seqno, format_interval_update(interval)));
+                          metric, seqno, format_interval_update(interval));
                 /* extra data? */
                 if((u_int)parsed_len < len)
                     subtlvs_print(ndo, message + 2 + parsed_len,
@@ -632,15 +686,15 @@ babel_print_v2(netdissect_options *ndo,
 
         case MESSAGE_REQUEST_SRC_SPECIFIC : {
             if(!ndo->ndo_vflag)
-                ND_PRINT((ndo, " ss-request"));
+                ND_PRINT(" ss-request");
             else {
                 int rc, parsed_len = 3;
                 u_char ae, plen, src_plen, prefix[16], src_prefix[16];
-                ND_PRINT((ndo, "\n\tSS-Request "));
+                ND_PRINT("\n\tSS-Request ");
                 if(len < 3) goto invalid;
-                ae = message[2];
-                plen = message[3];
-                src_plen = message[4];
+                ae = GET_U_1(message + 2);
+                plen = GET_U_1(message + 3);
+                src_plen = GET_U_1(message + 4);
                 rc = network_prefix(ae, plen, 0, message + 2 + parsed_len,
                                     NULL, len - parsed_len, prefix);
                 if(rc < 0) goto invalid;
@@ -654,10 +708,10 @@ babel_print_v2(netdissect_options *ndo,
                     src_plen += 96;
                 parsed_len += rc;
                 if(ae == 0) {
-                    ND_PRINT((ndo, "for any"));
+                    ND_PRINT("for any");
                 } else {
-                    ND_PRINT((ndo, "for (%s, ", format_prefix(ndo, prefix, plen)));
-                    ND_PRINT((ndo, "%s)", format_prefix(ndo, src_prefix, src_plen)));
+                    ND_PRINT("for (%s, ", format_prefix(ndo, prefix, plen));
+                    ND_PRINT("%s)", format_prefix(ndo, src_prefix, src_plen));
                 }
             }
         }
@@ -665,19 +719,19 @@ babel_print_v2(netdissect_options *ndo,
 
         case MESSAGE_MH_REQUEST_SRC_SPECIFIC : {
             if(!ndo->ndo_vflag)
-                ND_PRINT((ndo, " ss-mh-request"));
+                ND_PRINT(" ss-mh-request");
             else {
                 int rc, parsed_len = 14;
                 u_short seqno;
                 u_char ae, plen, src_plen, prefix[16], src_prefix[16], hopc;
                 const u_char *router_id = NULL;
-                ND_PRINT((ndo, "\n\tSS-MH-Request "));
+                ND_PRINT("\n\tSS-MH-Request ");
                 if(len < 14) goto invalid;
-                ae = message[2];
-                plen = message[3];
-                seqno = EXTRACT_16BITS(message + 4);
-                hopc = message[6];
-                src_plen = message[7];
+                ae = GET_U_1(message + 2);
+                plen = GET_U_1(message + 3);
+                seqno = GET_BE_U_2(message + 4);
+                hopc = GET_U_1(message + 6);
+                src_plen = GET_U_1(message + 7);
                 router_id = message + 8;
                 rc = network_prefix(ae, plen, 0, message + 2 + parsed_len,
                                     NULL, len - parsed_len, prefix);
@@ -690,30 +744,120 @@ babel_print_v2(netdissect_options *ndo,
                 if(rc < 0) goto invalid;
                 if(ae == 1)
                     src_plen += 96;
-                ND_PRINT((ndo, "(%u hops) for (%s, ",
-                          hopc, format_prefix(ndo, prefix, plen)));
-                ND_PRINT((ndo, "%s) seqno %u id %s",
+                ND_PRINT("(%u hops) for (%s, ",
+                          hopc, format_prefix(ndo, prefix, plen));
+                ND_PRINT("%s) seqno %u id %s",
                           format_prefix(ndo, src_prefix, src_plen),
-                          seqno, format_id(router_id)));
+                          seqno, format_id(ndo, router_id));
+            }
+        }
+            break;
+
+        case MESSAGE_MAC: {
+            if (!ndo->ndo_vflag)
+                ND_PRINT(" mac");
+            else {
+                ND_PRINT("\n\tMAC ");
+                ND_PRINT("len %u", len);
+            }
+        }
+            break;
+
+        case MESSAGE_PC: {
+            if (!ndo->ndo_vflag)
+                ND_PRINT(" pc");
+            else {
+                ND_PRINT("\n\tPC");
+                if(len < 4) goto invalid;
+                ND_PRINT(" value %u",
+                    GET_BE_U_4(message + 2));
+                ND_PRINT(" index len %u", len-4);
+            }
+        }
+            break;
+
+        case MESSAGE_CHALLENGE_REQUEST: {
+            if (!ndo->ndo_vflag)
+                ND_PRINT(" challenge_request");
+            else {
+                ND_PRINT("\n\tChallenge Request");
+                if(len > 192) goto invalid;
+                ND_PRINT(" len %u", len);
+            }
+        }
+            break;
+
+        case MESSAGE_CHALLENGE_REPLY: {
+            if (!ndo->ndo_vflag)
+                ND_PRINT(" challenge_reply");
+            else {
+                ND_PRINT("\n\tChallenge Reply");
+                if (len > 192) goto invalid;
+                ND_PRINT(" len %u", len);
             }
         }
             break;
 
         default:
             if (!ndo->ndo_vflag)
-                ND_PRINT((ndo, " unknown"));
+                ND_PRINT(" unknown");
             else
-                ND_PRINT((ndo, "\n\tUnknown message type %d", type));
+                ND_PRINT("\n\tUnknown message type %u", type);
         }
         i += len + 2;
     }
+
+    return 0; /* OK */
+
+trunc:
+    return -1; /* packet truncated by capture process */
+
+invalid:
+    return -2; /* packet is invalid */
+}
+
+static void
+babel_print_v2(netdissect_options *ndo,
+               const u_char *cp, u_int length)
+{
+    u_short bodylen;
+    int ret;
+
+    ND_TCHECK_4(cp);
+    if (length < 4)
+        goto invalid;
+    bodylen = GET_BE_U_2(cp + 2);
+    ND_PRINT(" (%u)", bodylen);
+    length -= 4;
+    cp += 4;
+
+    /* Process the TLVs in the body */
+    if (length < bodylen)
+        goto invalid;
+    ret = babel_print_v2_tlvs(ndo, cp, bodylen, length);
+    if (ret == -1)
+        goto trunc;
+    if (ret == -2)
+        goto invalid;
+    length -= bodylen;
+    cp += bodylen;
+
+    /* If there's a trailer, process the TLVs in the trailer */
+    if (length != 0) {
+	if(ndo->ndo_vflag) ND_PRINT("\n\t----");
+	else ND_PRINT(" |");
+        ret = babel_print_v2_tlvs(ndo, cp, length, length);
+        if (ret == -1)
+            goto trunc;
+        if (ret == -2)
+            goto invalid;
+    }
     return;
 
  trunc:
-    ND_PRINT((ndo, " %s", tstr));
+    nd_print_trunc(ndo);
     return;
 
  invalid:
-    ND_PRINT((ndo, "%s", istr));
-    return;
+    nd_print_invalid(ndo);
 }
