@@ -23,27 +23,27 @@
 /* \summary: ATM LANE printer */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
+#define ND_LONGJMP_FROM_TCHECK
 #include "netdissect.h"
 #include "extract.h"
-#include "ether.h"
 
 struct lecdatahdr_8023 {
-  uint16_t le_header;
-  uint8_t h_dest[ETHER_ADDR_LEN];
-  uint8_t h_source[ETHER_ADDR_LEN];
-  uint16_t h_type;
+  nd_uint16_t le_header;
+  nd_mac_addr h_dest;
+  nd_mac_addr h_source;
+  nd_uint16_t h_type;
 };
 
 struct lane_controlhdr {
-  uint16_t lec_header;
-  uint8_t lec_proto;
-  uint8_t lec_vers;
-  uint16_t lec_opcode;
+  nd_uint16_t lec_header;
+  nd_uint8_t  lec_proto;
+  nd_uint8_t  lec_vers;
+  nd_uint16_t lec_opcode;
 };
 
 static const struct tok lecop2str[] = {
@@ -69,15 +69,10 @@ static const struct tok lecop2str[] = {
 static void
 lane_hdr_print(netdissect_options *ndo, const u_char *bp)
 {
-	ND_PRINT((ndo, "lecid:%x ", EXTRACT_16BITS(bp)));
+	ND_PRINT("lecid:%x ", GET_BE_U_2(bp));
 }
 
 /*
- * This is the top level routine of the printer.  'p' points
- * to the LANE header of the packet, 'h->ts' is the timestamp,
- * 'h->len' is the length of the packet off the wire, and 'h->caplen'
- * is the number of bytes actually captured.
- *
  * This assumes 802.3, not 802.5, LAN emulation.
  */
 void
@@ -85,25 +80,24 @@ lane_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen)
 {
 	const struct lane_controlhdr *lec;
 
-	if (caplen < sizeof(struct lane_controlhdr)) {
-		ND_PRINT((ndo, "[|lane]"));
-		return;
-	}
+	ndo->ndo_protocol = "lane";
 
 	lec = (const struct lane_controlhdr *)p;
-	if (EXTRACT_16BITS(&lec->lec_header) == 0xff00) {
+	if (GET_BE_U_2(lec->lec_header) == 0xff00) {
 		/*
 		 * LE Control.
 		 */
-		ND_PRINT((ndo, "lec: proto %x vers %x %s",
-		    lec->lec_proto, lec->lec_vers,
-		    tok2str(lecop2str, "opcode-#%u", EXTRACT_16BITS(&lec->lec_opcode))));
+		ND_PRINT("lec: proto %x vers %x %s",
+			 GET_U_1(lec->lec_proto),
+			 GET_U_1(lec->lec_vers),
+			 tok2str(lecop2str, "opcode-#%u", GET_BE_U_2(lec->lec_opcode)));
 		return;
 	}
 
 	/*
 	 * Go past the LE header.
 	 */
+	ND_TCHECK_2(p); /* Needed */
 	length -= 2;
 	caplen -= 2;
 	p += 2;
@@ -114,11 +108,3 @@ lane_print(netdissect_options *ndo, const u_char *p, u_int length, u_int caplen)
 	 */
 	ether_print(ndo, p, length, caplen, lane_hdr_print, p - 2);
 }
-
-u_int
-lane_if_print(netdissect_options *ndo, const struct pcap_pkthdr *h, const u_char *p)
-{
-	lane_print(ndo, p, h->len, h->caplen);
-
-	return (sizeof(struct lecdatahdr_8023));
-}
