@@ -28,15 +28,14 @@
 /* \summary: ZeroMQ Message Transport Protocol (ZMTP) printer */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
-#include <netdissect-stdinc.h>
+#include "netdissect-stdinc.h"
 
 #include "netdissect.h"
 #include "extract.h"
 
-static const char tstr[] = " [|zmtp1]";
 
 /* Maximum number of ZMTP/1.0 frame body bytes (without the flags) to dump in
  * hex and ASCII under a single "-v" flag.
@@ -79,34 +78,33 @@ zmtp1_print_frame(netdissect_options *ndo, const u_char *cp, const u_char *ep)
 	uint64_t body_len_declared, body_len_captured, header_len;
 	uint8_t flags;
 
-	ND_PRINT((ndo, "\n\t"));
-	ND_TCHECK2(*cp, 1); /* length/0xFF */
+	ND_PRINT("\n\t");
 
-	if (cp[0] != 0xFF) {
+	if (GET_U_1(cp) != 0xFF) {	/* length/0xFF */
 		header_len = 1; /* length */
-		body_len_declared = cp[0];
-		ND_PRINT((ndo, " frame flags+body  (8-bit) length %" PRIu64, body_len_declared));
+		body_len_declared = GET_U_1(cp);
+		ND_PRINT(" frame flags+body  (8-bit) length %" PRIu64, body_len_declared);
 	} else {
 		header_len = 1 + 8; /* 0xFF, length */
-		ND_PRINT((ndo, " frame flags+body (64-bit) length"));
-		ND_TCHECK2(*cp, header_len); /* 0xFF, length */
-		body_len_declared = EXTRACT_64BITS(cp + 1);
-		ND_PRINT((ndo, " %" PRIu64, body_len_declared));
+		ND_PRINT(" frame flags+body (64-bit) length");
+		ND_TCHECK_LEN(cp, header_len); /* 0xFF, length */
+		body_len_declared = GET_BE_U_8(cp + 1);
+		ND_PRINT(" %" PRIu64, body_len_declared);
 	}
 	if (body_len_declared == 0)
 		return cp + header_len; /* skip to the next frame */
-	ND_TCHECK2(*cp, header_len + 1); /* ..., flags */
-	flags = cp[header_len];
+	ND_TCHECK_LEN(cp, header_len + 1); /* ..., flags */
+	flags = GET_U_1(cp + header_len);
 
 	body_len_captured = ep - cp - header_len;
 	if (body_len_declared > body_len_captured)
-		ND_PRINT((ndo, " (%" PRIu64 " captured)", body_len_captured));
-	ND_PRINT((ndo, ", flags 0x%02x", flags));
+		ND_PRINT(" (%" PRIu64 " captured)", body_len_captured);
+	ND_PRINT(", flags 0x%02x", flags);
 
 	if (ndo->ndo_vflag) {
-		uint64_t body_len_printed = min(body_len_captured, body_len_declared);
+		uint64_t body_len_printed = ND_MIN(body_len_captured, body_len_declared);
 
-		ND_PRINT((ndo, " (%s|%s|%s|%s|%s|%s|%s|%s)",
+		ND_PRINT(" (%s|%s|%s|%s|%s|%s|%s|%s)",
 			flags & 0x80 ? "MBZ" : "-",
 			flags & 0x40 ? "MBZ" : "-",
 			flags & 0x20 ? "MBZ" : "-",
@@ -114,38 +112,37 @@ zmtp1_print_frame(netdissect_options *ndo, const u_char *cp, const u_char *ep)
 			flags & 0x08 ? "MBZ" : "-",
 			flags & 0x04 ? "MBZ" : "-",
 			flags & 0x02 ? "MBZ" : "-",
-			flags & 0x01 ? "MORE" : "-"));
+			flags & 0x01 ? "MORE" : "-");
 
 		if (ndo->ndo_vflag == 1)
-			body_len_printed = min(VBYTES + 1, body_len_printed);
+			body_len_printed = ND_MIN(VBYTES + 1, body_len_printed);
 		if (body_len_printed > 1) {
-			ND_PRINT((ndo, ", first %" PRIu64 " byte(s) of body:", body_len_printed - 1));
+			ND_PRINT(", first %" PRIu64 " byte(s) of body:", body_len_printed - 1);
 			hex_and_ascii_print(ndo, "\n\t ", cp + header_len + 1, body_len_printed - 1);
-			ND_PRINT((ndo, "\n"));
 		}
 	}
 
 	/*
 	 * Do not advance cp by the sum of header_len and body_len_declared
-	 * before each offset has successfully passed ND_TCHECK2() as the
+	 * before each offset has successfully passed ND_TCHECK_LEN() as the
 	 * sum can roll over (9 + 0xfffffffffffffff7 = 0) and cause an
 	 * infinite loop.
 	 */
 	cp += header_len;
-	ND_TCHECK2(*cp, body_len_declared); /* Next frame within the buffer ? */
+	ND_TCHECK_LEN(cp, body_len_declared); /* Next frame within the buffer ? */
 	return cp + body_len_declared;
 
 trunc:
-	ND_PRINT((ndo, "%s", tstr));
-	return ep;
+	nd_trunc_longjmp(ndo);
 }
 
 void
 zmtp1_print(netdissect_options *ndo, const u_char *cp, u_int len)
 {
-	const u_char *ep = min(ndo->ndo_snapend, cp + len);
+	const u_char *ep = ND_MIN(ndo->ndo_snapend, cp + len);
 
-	ND_PRINT((ndo, ": ZMTP/1.0"));
+	ndo->ndo_protocol = "zmtp1";
+	ND_PRINT(": ZMTP/1.0");
 	while (cp < ep)
 		cp = zmtp1_print_frame(ndo, cp, ep);
 }
@@ -170,50 +167,48 @@ static const u_char *
 zmtp1_print_intermediate_part(netdissect_options *ndo, const u_char *cp, const u_int len)
 {
 	u_int frame_offset;
-	uint64_t remaining_len;
+	u_int remaining_len;
 
-	ND_TCHECK2(*cp, 2);
-	frame_offset = EXTRACT_16BITS(cp);
-	ND_PRINT((ndo, "\n\t frame offset 0x%04x", frame_offset));
+	frame_offset = GET_BE_U_2(cp);
+	ND_PRINT("\n\t frame offset 0x%04x", frame_offset);
 	cp += 2;
-	remaining_len = ndo->ndo_snapend - cp; /* without the frame length */
+	remaining_len = ND_BYTES_AVAILABLE_AFTER(cp); /* without the frame length */
 
 	if (frame_offset == 0xFFFF)
 		frame_offset = len - 2; /* always within the declared length */
 	else if (2 + frame_offset > len) {
-		ND_PRINT((ndo, " (exceeds datagram declared length)"));
+		ND_PRINT(" (exceeds datagram declared length)");
 		goto trunc;
 	}
 
 	/* offset within declared length of the datagram */
 	if (frame_offset) {
-		ND_PRINT((ndo, "\n\t frame intermediate part, %u bytes", frame_offset));
+		ND_PRINT("\n\t frame intermediate part, %u bytes", frame_offset);
 		if (frame_offset > remaining_len)
-			ND_PRINT((ndo, " (%"PRIu64" captured)", remaining_len));
+			ND_PRINT(" (%u captured)", remaining_len);
 		if (ndo->ndo_vflag) {
-			uint64_t len_printed = min(frame_offset, remaining_len);
+			u_int len_printed = ND_MIN(frame_offset, remaining_len);
 
 			if (ndo->ndo_vflag == 1)
-				len_printed = min(VBYTES, len_printed);
+				len_printed = ND_MIN(VBYTES, len_printed);
 			if (len_printed > 1) {
-				ND_PRINT((ndo, ", first %"PRIu64" byte(s):", len_printed));
+				ND_PRINT(", first %u byte(s):", len_printed);
 				hex_and_ascii_print(ndo, "\n\t ", cp, len_printed);
-				ND_PRINT((ndo, "\n"));
 			}
 		}
 	}
 	return cp + frame_offset;
 
 trunc:
-	ND_PRINT((ndo, "%s", tstr));
-	return cp + len;
+	nd_trunc_longjmp(ndo);
 }
 
 void
-zmtp1_print_datagram(netdissect_options *ndo, const u_char *cp, const u_int len)
+zmtp1_datagram_print(netdissect_options *ndo, const u_char *cp, const u_int len)
 {
-	const u_char *ep = min(ndo->ndo_snapend, cp + len);
+	const u_char *ep = ND_MIN(ndo->ndo_snapend, cp + len);
 
+	ndo->ndo_protocol = "zmtp1";
 	cp = zmtp1_print_intermediate_part(ndo, cp, len);
 	while (cp < ep)
 		cp = zmtp1_print_frame(ndo, cp, ep);
